/*
 * Lands tree container
 *
 * Copyright(c) 2008 olyutorskii
 * $Id: LandsTree.java 231 2008-10-07 18:24:53Z olyutorskii $
 */

package jp.sourceforge.jindolf;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * 国一覧Tree周辺コンポーネント群
 */
@SuppressWarnings("serial")
public class LandsTree
        extends JPanel
        implements ActionListener, TreeSelectionListener{
    
    private static final String ascendingTip = "押すと降順に";
    private static final String descendingTip = "押すと昇順に";
    private static final String orderTip = "選択中の国の村一覧を読み込み直す";
    private static final ImageIcon ascendIcon;
    private static final ImageIcon descendIcon;
    
    static{
        URL url;
        url = Jindolf.getResource("resources/image/ascend.png");
        ascendIcon = new ImageIcon(url);
        url = Jindolf.getResource("resources/image/descend.png");
        descendIcon = new ImageIcon(url);
    }
    
    private final JButton orderButton = new JButton();
    private final JButton reloadButton = new JButton();
    private final JTree treeView = new JTree();

    private boolean ascending = false;
    
    public LandsTree(){
        super();

        design();
        
        this.orderButton.setIcon(descendIcon);
        this.orderButton.setToolTipText(descendingTip);
        this.orderButton.setMargin(new Insets(1, 1, 1, 1));
        this.orderButton.setActionCommand(ActionManager.COMMAND_SWITCHORDER);
        this.orderButton.addActionListener(this);

        URL url;
        url = Jindolf.getResource("resources/image/reload.png");
        this.reloadButton.setIcon(new ImageIcon(url));
        this.reloadButton.setToolTipText(orderTip);
        this.reloadButton.setMargin(new Insets(1, 1, 1, 1));
        this.reloadButton.setActionCommand(ActionManager.COMMAND_VILLAGELIST);

        TreeSelectionModel selModel = this.treeView.getSelectionModel();
        selModel.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        this.treeView.addTreeSelectionListener(this);
        
        return;
    }

    /**
     * GUIパーツのデザインを行う。
     */
    private void design(){
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);

        constraints.insets = new Insets(1, 1, 1, 1);

        constraints.weightx = 0.0;
        constraints.weighty = 0.0;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.NONE;
        add(this.orderButton, constraints);

        constraints.gridwidth = GridBagConstraints.REMAINDER;
        add(this.reloadButton, constraints);

        constraints.insets = new Insets(0, 0, 0, 0);

        constraints.gridwidth = GridBagConstraints.REMAINDER;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
        add(createLandSelector(), constraints);
        
        return;
    }
    
    /**
     * 国村選択ツリーコンポーネントを生成する。
     * @return 国村選択ツリーコンポーネント
     */
    private JComponent createLandSelector(){
        this.treeView.setRootVisible(false);
        this.treeView.setCellRenderer(new VillageIconRenderer());
        
        EmptyBorder border = new EmptyBorder(5, 5, 5, 5);
        this.treeView.setBorder(border);

        JScrollPane landSelector = new JScrollPane(this.treeView);

        return landSelector;
    }

    /**
     * リロードボタンを返す。
     * @return リロードボタン
     */
    public JButton getReloadVillageListButton(){
        return this.reloadButton;
    }
    
    /**
     * 国村選択ツリービューを返す。
     * @return 国村選択ツリービュー
     */
    public JTree getTreeView(){
        return this.treeView;
    }

    /**
     * 指定した国を展開する。
     * @param land 国
     */
    public void expandLand(Land land){
        TreeModel model = this.treeView.getModel();
        Object root = model.getRoot();
        Object[] path = {root, land};
        TreePath treePath = new TreePath(path);
        this.treeView.expandPath(treePath);
        return;
    }
    
    /**
     * 管理下のLandsModelを返す。
     * @return LandsModel
     */
    private LandsModel getLandsModel(){
        TreeModel model = this.treeView.getModel();
        if(model instanceof LandsModel){
            return (LandsModel) model;
        }
        return null;
    }
    
    /**
     * Tree表示順を反転させる。
     * @return 反転後が昇順ならtrue
     */
    private boolean toggleTreeOrder(){
        this.ascending = ! this.ascending;

        if(this.ascending){
            this.orderButton.setToolTipText(ascendingTip);
            this.orderButton.setIcon(ascendIcon);
        }else{
            this.orderButton.setToolTipText(descendingTip);
            this.orderButton.setIcon(descendIcon);
        }

        final TreePath lastPath = this.treeView.getSelectionPath();
        
        LandsModel model = getLandsModel();
        if(model != null){
            model.setAscending(this.ascending);
        }
        
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                if(lastPath != null){
                    treeView.setSelectionPath(lastPath);
                    treeView.scrollPathToVisible(lastPath);
                }
                return;
            }
        });
        
        return this.ascending;
    }
    
    /**
     * ボタン押下処理
     * @param event ボタン押下イベント
     */
    public void actionPerformed(ActionEvent event){
        String cmd = event.getActionCommand();
        if(ActionManager.COMMAND_SWITCHORDER.equals(cmd)){
            toggleTreeOrder();
        }
        return;
    }
    
    /**
     * ツリーリストで何らかの要素（国、村）がクリックされたときの処理。
     * @param event イベント
     */
    public void valueChanged(TreeSelectionEvent event){
        TreePath path = event.getNewLeadSelectionPath();
        if(path == null){
            this.reloadButton.setEnabled(false);
            return;
        }

        Object selObj = path.getLastPathComponent();
        
        if( selObj instanceof Land ){
            this.reloadButton.setEnabled(true);
        }else{
            this.reloadButton.setEnabled(false);
        }

        return;
    }
    
    /**
     * L&F を変更する。
     */
    @Override
    public void updateUI(){
        super.updateUI();
        if(this.treeView != null){
            this.treeView.setCellRenderer(new VillageIconRenderer());
        }
        return;
    }
}
